/****************************************************************************
**
**  indeview_slide.h     of the IndeView project, see http://www.indeview.org
**
**
**  The IndeView application is a stand-alone viewer to display presentations
**  created by IndeView creator modules like IndeViewExport.bas or kpr2iv.sh.
**  -------------------------------------------------------------------------
**  This enables you to show OpenOffice/StarOffice or KOffice presentations
**  on machines where the respective presentation program is not available.
**
**
**  Copyright (C) 2003..2004 Michael & Karl-Heinz Zimmer.
**  All rights reserved.
**
**  This file is distributed under the terms of the
**  GNU Lesser General Public License version 2.1 as published by the
**  Free Software Foundation and appearing in the file LICENSE included
**  in the packaging of this file.
**
**  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
**  THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
**  PURPOSE.
**
**  See http://www.indeview.org for further information.
**
**  Contact info@indeview.org if any conditions of this
**  licensing are not clear to you.
**
**
**  Please consider sending
**  a little picture postcard to   Michael & Karl-Heinz Zimmer
**                                 Butzengasse 15
**                                 54343 Fhren
**                                 Germany
**
**  Your feedback will be very much appreciated: we would be glad about
**  receiving a visual representation of where on earth you are living
**  using our little module.
**
*****************************************************************************/

#ifndef INDEVIEW_SLIDE_H
#define INDEVIEW_SLIDE_H

#include <qpixmap.h>
#include <qstring.h>
#include <qstringlist.h>

#include "indeview_enums.h"


class IndeViewSizeAndFile
{
public:
  IndeViewSizeAndFile()
  : mSize( QSize() ),
    mFileName( "" ){}
  IndeViewSizeAndFile(const QString& fileName, QSize size)
  : mSize( size ),
    mFileName( fileName ){}
  void set(const QString& fileName, QSize size)
  {
    mSize = size;
    mFileName = fileName;
  }
  void reset()
  {
    mSize = QSize();
    mFileName = QString();
  }
  QSize   size()     const { return mSize; }
  QString fileName() const { return mFileName; }

  bool isEmpty() const
  {
    return mFileName.isEmpty() || mSize.isEmpty();
  }

  bool operator==( const IndeViewSizeAndFile& it ) const
  {
    return (it.size() == mSize) && !mFileName.compare( it.fileName() );
  }
private:
  QSize   mSize;
  QString mFileName;
};

typedef QValueList<IndeViewSizeAndFile> IndeViewImageList;


#define INDEVIEW_N_CACHED_PIXMAPS 7

struct IndeViewCachedPixmap
{
  IndeViewCachedPixmap& operator=( const IndeViewCachedPixmap& R );
  IndeViewSizeAndFile info;
  QPixmap pix;
};

class IndeViewCachedPixmaps
{
public:
  void set(int i, const IndeViewSizeAndFile& _info, const QPixmap& _pix);
  IndeViewCachedPixmap list[ INDEVIEW_N_CACHED_PIXMAPS ];
};


class IndeViewSlide
{
public:
  IndeViewSlide( const QString& pageID, IndeViewCachedPixmaps& cache )
    : pNext(0),
      pPrev(0),
      sPageID( pageID ),
      sPageName(""),
      sFileName(""),
      sNextID(""),
      eChange( IndeViewChangeMode_USER ),
      nDuration(10),
      eDurationMode( IndeViewDurationMode_RELATIVE ),
      eFadeEffect( IndeViewFadeEffect_NONE ),
      nAnimationSpeed(5),
      sFadeSound(""),
      eFadeSoundMode( IndeViewFadeSoundMode_FULL ),
      sViewSound(""),
      nViewSoundLoop(1),
      rCache(cache){}
  ~IndeViewSlide()
  {
    // do nothing
  }

  // Read image from file or use the cached image.
  // Note: the image must exactly match the given size!
  QPixmap* getImage(QSize size, bool justPreloading=false)const;
  void addImageDescription(const IndeViewSizeAndFile& description);

  // obligatory settings:
  IndeViewSlide*  pNext;
  IndeViewSlide*  pPrev;
  QString         sPageID;   // the internal name of this slide
  QString         sPageName; // the descriptive name of this slide
  QString         sFileName; // the name of this slide's image file
  QString         sNextID;   // the internal name of the following slide

  // optional settings:
  QStringList           lNotes; // 0 or 1 or many lines (in the run file each one is prepended by "notes=" )
  IndeViewChangeMode    eChange; // "USER" (if com.sun.star.presentation.DrawPage is 0 or 2), "AUTO" (if 1)
  int                   nDuration;       // only valid if maChange == "AUTO": the time in tenths of a second the page is shown, before switching to the next page
  IndeViewDurationMode  eDurationMode; // if 'relative' (default) and nViewSoundLoop is not 0 then duration == 10th of a second after view sound has finished playing
  IndeViewFadeEffect    eFadeEffect;     // see com.sun.star.presentation.FadeEffect
  int                   nAnimationSpeed; // speed of the fade-in effect of this page: 1: "SLOW", 5: "MEDIUM", 9: "FAST"
  QString               sFadeSound; // sound to be played once during the fade-in effect of this page
  IndeViewFadeSoundMode eFadeSoundMode; // 'full' (default) == play the entire sound, 'cut' == stop playing when slide is completely visible
  QString               sViewSound; // sound to be played once or repeatedly when viewing this page
  int                   nViewSoundLoop; // determines how often the viewing sound will be played, if 0 it will be played continuously
  IndeViewImageList     images;

private:
  IndeViewCachedPixmaps& rCache;
};


/* XPM */
static const char * INDEVIEWERROR_Xpm[] = {
"200 200 79 1",
" 	c None",
".	c #000000",
"+	c #493B00",
"@	c #8C7400",
"#	c #B99E00",
"$	c #D7BD00",
"%	c #EAD400",
"&	c #F6E400",
"*	c #FBEF00",
"=	c #FEF500",
"-	c #FFF900",
";	c #FFFA00",
">	c #655300",
",	c #E9D200",
"'	c #FDF400",
")	c #7C6700",
"!	c #D5BC00",
"~	c #FCF000",
"{	c #463900",
"]	c #C3A900",
"^	c #FBED00",
"/	c #6A5700",
"(	c #DFC600",
"_	c #E4CD00",
":	c #382D00",
"<	c #AE9400",
"[	c #FEF700",
"}	c #4E4000",
"|	c #9E8500",
"1	c #CFB500",
"2	c #261E00",
"3	c #5B4A00",
"4	c #FCF700",
"5	c #D5D100",
"6	c #B2AF00",
"7	c #939000",
"8	c #787600",
"9	c #605E00",
"0	c #4C4B00",
"a	c #3B3A00",
"b	c #2C2B00",
"c	c #212000",
"d	c #171700",
"e	c #0F0F00",
"f	c #0A0A00",
"g	c #060600",
"h	c #030300",
"i	c #010100",
"j	c #C3BF00",
"k	c #848100",
"l	c #4A4900",
"m	c #151500",
"n	c #ACA900",
"o	c #525000",
"p	c #9D9A00",
"q	c #282700",
"r	c #FAF500",
"s	c #6A6800",
"t	c #898600",
"u	c #4F4D00",
"v	c #191900",
"w	c #414000",
"x	c #666400",
"y	c #858200",
"z	c #A19E00",
"A	c #B8B400",
"B	c #CBC700",
"C	c #DBD700",
"D	c #E7E200",
"E	c #F1EC00",
"F	c #F8F300",
"G	c #0E0E00",
"H	c #615F00",
"I	c #AEAB00",
"J	c #F5F000",
"K	c #555300",
"L	c #AAA700",
"M	c #131300",
"N	c #8C8900",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
".........................................................................................................................................+@#$%&*=-;-=*&%$#@+............................................",
".....................................................................................................................................>#,';;;;;;;;;;;;;;;;;;;',#>........................................",
"..................................................................................................................................)!~;;;;;;;;;;;;;;;;;;;;;;;;;;;~!).....................................",
"...............................................................................................................................{]^;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;^]{..................................",
"............................................................................................................................./(;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;(/................................",
"...........................................................................................................................>_;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;_>..............................",
".........................................................................................................................:$;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;$:............................",
"........................................................................................................................<[;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;[<...........................",
"......................................................................................................................}%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;%}.........................",
".....................................................................................................................|-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-|........................",
"....................................................................................................................1;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;1.......................",
"..................................................................................................................2,;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,2.....................",
".................................................................................................................+&;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;&+....................",
"................................................................................................................3^;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;^3...................",
"...............................................................................................................3~;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;~3..................",
"..............................................................................................................+^;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;^+.................",
".............................................................................................................2&;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;&2................",
".............................................................................................................,;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,................",
"............................................................................................................1;;;;;;;;;;;;;.........;;;;;;;;;;;;;;;;;;;;;;;;;;;.........;;;;;;;;;;;;;;;;;1...............",
"...........................................................................................................|;;;;;;;;;;;;;...........;;;;;;;;;;;;;;;;;;;;;;;;;...........;;;;;;;;;;;;;;;;;|..............",
"..........................................................................................................}-;;;;;;;;;;;...............;;;;;;;;;;;;;;;;;;;;;...............;;;;;;;;;;;;;;;-}.............",
"..........................................................................................................%;;;;;;;;;;;;...............;;;;;;;;;;;;;;;;;;;;;...............;;;;;;;;;;;;;;;;%.............",
".........................................................................................................<;;;;;;;;;;;;.................;;;;;;;;;;;;;;;;;;;.................;;;;;;;;;;;;;;;;<............",
"........................................................................................................:[;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;[:...........",
"........................................................................................................$;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;$...........",
".......................................................................................................>;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;>..........",
"......................................................................................................._;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;_..........",
"....................................................................................................../;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;;/.........",
"......................................................................................................(;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;;(.........",
".....................................................................................................{;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;;;{........",
".....................................................................................................];;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;;;]........",
".....................................................................................................^;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;...................;;;;;;;;;;;;;;;;;;;^........",
"....................................................................................................);;;;;;;;;;;;;;;;;.................;;;;;;;;;;;;;;;;;;;.................;;;;;;;;;;;;;;;;;;;;;).......",
"....................................................................................................!;;;;;;;;;;;;;;;;;;...............;;;;;;;;;;;;;;;;;;;;;...............;;;;;;;;;;;;;;;;;;;;;;!.......",
"....................................................................................................~;;;;;;;;;;;;;;;;;;...............;;;;;;;;;;;;;;;;;;;;;...............;;;;;;;;;;;;;;;;;;;;;;~.......",
"...................................................................................................>;;;;;;;;;;;;;;;;;;;;;...........;;;;;;;;;;;;;;;;;;;;;;;;;...........;;;;;;;;;;;;;;;;;;;;;;;;;>......",
"...................................................................................................#;;;;;;;;;;;;;;;;;;;;;;.........;;;;;;;;;;;;;;;;;;;;;;;;;;;.........;;;;;;;;;;;;;;;;;;;;;;;;;;#......",
"...................................................................................................,;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,......",
"...................................................................................................';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;'......",
"..................................................................................................+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;+.....",
"..................................................................................................@;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;@.....",
"..................................................................................................#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;#.....",
"..................................................................................................$;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;$.....",
"..................................................................................................%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;%.....",
"..................................................................................................&;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;&.....",
"..................................................................................................*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*.....",
"..................................................................................................=;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;=.....",
"..................................................................................................-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-.....",
"..................................................................................................;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;.....",
"..................................................................................................-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-.....",
"..................................................................................................=;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;=.....",
"..................................................................................................*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*.....",
"..................................................................................................&;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;&.....",
"..................................................................................................%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;%.....",
"..................................................................................................$;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;$.....",
"..................................................................................................#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;#.....",
"..................................................................................................@;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;@.....",
"..................................................................................................+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;+.....",
"...................................................................................................';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;'......",
"...................................................................................................,;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;4567890abcdefghi.ihgfedcba0987654;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,......",
"...................................................................................................#;;;;;;;;;;;;;;;;;;;;;;;;jklm.....................................mlkj;;;;;;;;;;;;;;;;;;;;;;;;#......",
"...................................................................................................>;;;;;;;;;;;;;;;;;;;no...................................................on;;;;;;;;;;;;;;;;;;;>......",
"....................................................................................................~;;;;;;;;;;;;;;pq...........................................................qp;;;;;;;;;;;;;;~.......",
"....................................................................................................!;;;;;;;;;;rs...................................................................sr;;;;;;;;;;!.......",
"....................................................................................................);;;;;;;;t.........................................................................t;;;;;;;;).......",
".....................................................................................................^;;;;;u...........................................................................;;;;;;;;^........",
".....................................................................................................];;;;;;..........................................................................;;;;;;;;;]........",
".....................................................................................................{;;;;;;;..................vwxyzABCDEF4;;;;;;;;;;;;;4FEDCBAzyxwv..................;;;;;;;;;{........",
"......................................................................................................(;;;;;;.........GHIJ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;JIHG........K;;;;;;;;(.........",
"....................................................................................................../;;;;;;L..MN;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;NM..;;;;;;;;;/.........",
"......................................................................................................._;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;_..........",
".......................................................................................................>;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;>..........",
"........................................................................................................$;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;$...........",
"........................................................................................................:[;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;[:...........",
".........................................................................................................<;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<............",
"..........................................................................................................%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;%.............",
"..........................................................................................................}-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-}.............",
"...........................................................................................................|;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;|..............",
"............................................................................................................1;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;1...............",
".............................................................................................................,;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,................",
".............................................................................................................2&;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;&2................",
"..............................................................................................................+^;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;^+.................",
"...............................................................................................................3~;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;~3..................",
"................................................................................................................3^;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;^3...................",
".................................................................................................................+&;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;&+....................",
"..................................................................................................................2,;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,2.....................",
"....................................................................................................................1;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;1.......................",
".....................................................................................................................|-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-|........................",
"......................................................................................................................}%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;%}.........................",
"........................................................................................................................<[;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;[<...........................",
".........................................................................................................................:$;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;$:............................",
"...........................................................................................................................>_;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;_>..............................",
"............................................................................................................................./(;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;(/................................",
"...............................................................................................................................{]^;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;^]{..................................",
"..................................................................................................................................)!~;;;;;;;;;;;;;;;;;;;;;;;;;;;~!).....................................",
".....................................................................................................................................>#,';;;;;;;;;;;;;;;;;;;',#>........................................",
".........................................................................................................................................+@#$%&*=-;-=*&%$#@+............................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................",
"........................................................................................................................................................................................................"};


#endif // INDEVIEW_SLIDE_H
